@@ -1,3 +1,17 @@
+"""Session creation and datasets registration
+
+The RegistrationClient provides an high-level API for creating experimentation sessions on Alyx
+and registering associated datasets.
+
+Summary of methods
+------------------
+create_new_session - Create a new local session folder and optionally create session record on Alyx
+create_session - Create session record on Alyx from local path, without registering files
+create_sessions - Create sessions and register files for folder containing a given flag file
+register_session - Create a session on Alyx from local path and register any ALF datasets present
+register_files - Register a list of files to their respective sessions on Alyx
+"""
+import pathlib
 import uuid
 from pathlib import Path, PurePosixPath
 import datetime
@@ -5,13 +19,14 @@ import logging
 from uuid import UUID
 import itertools
 from collections import defaultdict
+from fnmatch import fnmatch
 
 import requests.exceptions
 
 from iblutil.io import hashfile
 
 import one.alf.io as alfio
-from one.alf.files import rel_path_parts, session_path_parts, get_session_path, get_alf_path
+from one.alf.files import session_path_parts, get_session_path
 import one.alf.exceptions as alferr
 from one.api import ONE
 from one.util import ensure_list
@@ -19,84 +34,6 @@ from one.util import ensure_list
 _logger = logging.getLogger(__name__)
 
 
-def register_dataset(file_list, one=None, created_by=None, repository=None, server_only=False,
-                     versions=None, default=True, dry=False, max_md5_size=None):
-    """
-    Registers a set of files belonging to a session only on the server
-
-    Parameters
-    ----------
-    file_list : list, str, pathlib.Path
-        A filepath (or list thereof) of ALF datasets to register to Alyx
-    one : one.api.OneAlyx
-        Optional one object, will create a default instance if none provided
-    created_by : str
-        Name of Alyx user (defaults to whoever is logged in to ONE instance)
-    repository : str
-        Name of the repository in Alyx to register to
-    server_only : bool
-        Will only create file records in the 'online' repositories and skips local repositories
-    versions : list of str
-        Optional version tags
-    default : bool
-        Whether to set as default revision (defaults to True)
-    dry : bool
-        When true returns POST data for registration endpoint without submitting the data
-    max_md5_size : int
-        Maximum file in bytes to compute md5 sum (always compute if None)
-
-    Returns
-    -------
-        A list of newly created Alyx dataset records or the registration data if dry == true
-    """
-    if created_by is None:
-        created_by = getattr(one.alyx, 'user', None)
-    if file_list is None or file_list == '' or file_list == []:
-        return
-    elif not isinstance(file_list, list):
-        file_list = [Path(file_list)]
-
-    assert len(set(get_session_path(f) for f in file_list)) == 1, 'Multiple sessions in list'
-
-    assert all([Path(f).exists() for f in file_list])
-    if isinstance(versions, str):
-        versions = [versions for _ in file_list]
-    assert isinstance(versions, list) and len(versions) == len(file_list)
-
-    # Determine revisions from file paths
-    revisions = [rel_path_parts(x, assert_valid=False)[1] for x in file_list]
-
-    # computing the md5 can be very long, so this is an option to skip if the file is bigger
-    # than a certain threshold
-    if max_md5_size:
-        hashes = [hashfile.md5(p) if
-                  p.stat().st_size < max_md5_size else None for p in file_list]
-    else:
-        hashes = [hashfile.md5(p) for p in file_list]
-
-    session_path = get_session_path(file_list[0])
-
-    # first register the file
-    r = {'created_by': created_by,
-         'path': session_path.relative_to((session_path.parents[2])).as_posix(),
-         'filenames': [p.relative_to(session_path).as_posix() for p in file_list],
-         'name': repository,
-         'server_only': server_only,
-         'hashes': hashes,
-         'filesizes': [p.stat().st_size for p in file_list],
-         'versions': versions,
-         'revisions': revisions,
-         'default': default}
-    if not dry:
-        one = one or ONE()
-        response = one.alyx.rest('register-file', 'create', data=r, no_cache=True)
-        for p in file_list:
-            _logger.info(f"ALYX REGISTERED DATA: {p}")
-        return response
-    else:
-        return r
-
-
 class RegistrationClient:
     """
     Object that keeps the ONE instance and provides method to create sessions and register data.
@@ -126,24 +63,37 @@ class RegistrationClient:
 
         Returns
         -------
-            TODO document return values
+        list of pathlib.Paths
+            Newly created session paths
+        list of dicts
+            Alyx session records
         """
-        flag_files = Path(root_data_folder).glob(glob_pattern)
+        flag_files = list(Path(root_data_folder).glob(glob_pattern))
+        records = []
         for flag_file in flag_files:
             if dry:
-                print(flag_file)
+                records.append(print(flag_file))
                 continue
             _logger.info('creating session for ' + str(flag_file.parent))
             # providing a false flag stops the registration after session creation
-            self.create_session(flag_file.parent)
+            records.append(self.create_session(flag_file.parent))
             flag_file.unlink()
-        return [ff.parent for ff in flag_files]
+        return [ff.parent for ff in flag_files], records
 
-    def create_session(self, session_path):
+    def create_session(self, session_path) -> dict:
+        """Create a remote session on Alyx from a local session path, without registering files
+
+        Parameters
+        ----------
+        session_path : str, pathlib.Path
+            The path ending with subject/date/number
+
+        Returns
+        -------
+        dict
+            Newly created session record
         """
-        create_session(session_path)
-        """
-        return self.register_session(session_path, file_list=False)
+        return self.register_session(session_path, file_list=False)[0]
 
     def create_new_session(self, subject, session_root=None, date=None, register=True):
         """Create a new local session folder and optionally create session record on Alyx
@@ -162,16 +112,24 @@ class RegistrationClient:
 
         Returns
         -------
-        New local session path; the experiment UUID if register is true
+        pathlib.Path
+            New local session path
+        str
+            The experiment UUID if register is True
 
         Examples
         --------
-        # Create a local session only
-        session_path, _ = RegistrationClient.create_new_session('Ian', register=False)
-        # Register a session on Alyx in a specific location
-        session_path, eid = RegistrationClient.create_new_session('Ian', '/data/mylab/Subjects')
-        # Create a session for a given date
-        session_path, eid = RegistrationClient.create_new_session('Ian', date='2020-01-01')
+        Create a local session only
+
+        >>> session_path, _ = RegistrationClient().create_new_session('Ian', register=False)
+
+        Register a session on Alyx in a specific location
+
+        >>> session_path, eid = RegistrationClient().create_new_session('Sy', '/data/lab/Subjects')
+
+        Create a session for a given date
+
+        >>> session_path, eid = RegistrationClient().create_new_session('Ian', date='2020-01-01')
         """
         assert not self.one.offline, 'ONE must be in online mode'
         date = self.ensure_ISO8601(date)  # Format, validate
@@ -194,7 +152,8 @@ class RegistrationClient:
 
         Returns
         -------
-        Iterable of file paths that match the dataset type patterns in Alyx
+        generator
+            Iterable of file paths that match the dataset type patterns in Alyx
         """
         session_path = Path(session_path)
         types = (x['filename_pattern'] for x in self.dtypes if x['filename_pattern'])
@@ -214,9 +173,9 @@ class RegistrationClient:
 
         Examples
         --------
-        client.assert_exists('ALK_036', 'subjects')
-        client.assert_exists('user_45', 'users')
-        client.assert_exists('local_server', 'repositories')
+        >>> client.assert_exists('ALK_036', 'subjects')
+        >>> client.assert_exists('user_45', 'users')
+        >>> client.assert_exists('local_server', 'repositories')
 
         Raises
         -------
@@ -231,7 +190,7 @@ class RegistrationClient:
             try:
                 self.one.alyx.rest(endpoint, 'read', id=str(member), no_cache=True)
             except requests.exceptions.HTTPError as ex:
-                if '404' not in str(ex):
+                if ex.response.status_code != 404:
                     raise ex
                 elif endpoint == 'subjects':
                     raise alferr.AlyxSubjectNotFound(member)
@@ -252,7 +211,8 @@ class RegistrationClient:
 
         Returns
         -------
-        The datetime as an ISO 8601 str
+        str
+            The datetime as an ISO 8601 string
         """
         date = date or datetime.datetime.now()  # If None get current time
         if isinstance(date, str):
@@ -265,6 +225,9 @@ class RegistrationClient:
         """
         Register session in Alyx
 
+        NB: If providing a lab or start_time kwarg, they must match the lab (if there is one)
+        and date of the session path.
+
         Parameters
         ----------
         ses_path : str, pathlib.Path
@@ -290,10 +253,36 @@ class RegistrationClient:
             The experiment type, e.g. 'Experiment', 'Base'
         task_protocol : str
             The task protocol (optional)
+        lab : str
+            The name of the lab where the session took place.  If None the lab name will be
+            taken from the path.  If no lab name is found in the path (i.e. no <lab>/Subjects)
+            the default lab on Alyx will be used.
+        start_time : str, datetime.datetime
+            The precise start time of the session.  The date must match the date in the session
+            path.
+        end_time : str, datetime.datetime
+            The precise end time of the session.
 
         Returns
         -------
-        TODO Document
+        dict
+            An Alyx session record
+        list, None
+            Alyx file records (or None if file_list is False)
+
+        Raises
+        ------
+        AssertionError
+            Subject does not exist on Alyx or provided start_time does not match date in
+            session path.
+        ValueError
+            The provided lab name does not match the one found in the session path or
+            start_time/end_time is not a valid ISO date time.
+        requests.HTTPError
+            A 400 status code means the submitted data was incorrect (e.g. task_protocol was an
+            int instead of a str); A 500 status code means there was a server error.
+        ConnectionError
+            Failed to connect to Alyx, most likely due to a bad internet connection.
         """
         if isinstance(ses_path, str):
             ses_path = Path(ses_path)
@@ -316,16 +305,20 @@ class RegistrationClient:
                 'number': details['number']}
         if kwargs.get('end_time', False):
             ses_['end_time'] = self.ensure_ISO8601(kwargs.pop('end_time'))
-        start_time = kwargs.pop('start_time', None)
+        start_time = self.ensure_ISO8601(kwargs.pop('start_time', details['date']))
+        assert start_time[:10] == details['date'], 'start_time doesn\'t match session path'
         if kwargs.get('procedures', False):
             ses_['procedures'] = ensure_list(kwargs.pop('procedures'))
         assert ('subject', 'number') not in kwargs
         if 'lab' not in kwargs and details['lab']:
             kwargs.update({'lab': details['lab']})
+        elif details['lab'] and kwargs.get('lab', details['lab']) != details['lab']:
+            names = (kwargs['lab'], details['lab'])
+            raise ValueError('lab kwarg "%s" does not match lab name in path ("%s")' % names)
         ses_.update(kwargs)
 
         if not session:  # Create from scratch
-            ses_['start_time'] = self.ensure_ISO8601(start_time)
+            ses_['start_time'] = start_time
             session = self.one.alyx.rest('sessions', 'create', data=ses_)
         else:  # Update existing
             if start_time:
@@ -334,55 +327,106 @@ class RegistrationClient:
 
         _logger.info(session['url'] + ' ')
         # at this point the session has been created. If create only, exit
-        if file_list:
-            self.register_files(self.find_files(ses_path) if file_list is True else file_list)
+        if not file_list:
+            return session, None
+        recs = self.register_files(self.find_files(ses_path) if file_list is True else file_list)
+        if recs:  # Update local session data after registering files
+            session['data_dataset_session_related'] = ensure_list(recs)
+        return session, recs
 
-        return session
+    def register_files(self, file_list, versions=None, default=True, created_by=None,
+                       server_only=False, repository=None, dry=False, max_md5_size=None):
+        """
+        Registers a set of files belonging to a session only on the server
 
-    def register_files(self, file_list):
+        Parameters
+        ----------
+        file_list : list, str, pathlib.Path
+            A filepath (or list thereof) of ALF datasets to register to Alyx
+        created_by : str
+            Name of Alyx user (defaults to whoever is logged in to ONE instance)
+        repository : str
+            Name of the repository in Alyx to register to
+        server_only : bool
+            Will only create file records in the 'online' repositories and skips local repositories
+        versions : list of str
+            Optional version tags
+        default : bool
+            Whether to set as default revision (defaults to True)
+        dry : bool
+            When true returns POST data for registration endpoint without submitting the data
+        max_md5_size : int
+            Maximum file in bytes to compute md5 sum (always compute if None)
+
+        Returns
+        -------
+        list of dicts, dict
+            A list of newly created Alyx dataset records or the registration data if dry
+        """
         F = defaultdict(list)  # empty map whose keys will be session paths
-        md5s = []
-        file_sizes = []
-        file_list = list(file_list)
+        V = defaultdict(list)  # empty map for versions
+        if isinstance(file_list, (str, pathlib.Path)):
+            file_list = [file_list]
+
+        if versions is None or isinstance(versions, str):
+            versions = itertools.repeat(versions)
+        else:
+            versions = itertools.cycle(versions)
 
         # Filter valid files and sort by session
-        for fn in file_list:
+        for fn, ver in zip(map(pathlib.Path, file_list), versions):
             session_path = get_session_path(fn)
             if fn.suffix not in self.file_extensions:
-                _logger.debug(f'{fn}: No matching extension {fn.suffix} in database')
+                _logger.debug(f'{fn}: No matching extension "{fn.suffix}" in database')
                 continue
-            if fn.suffix not in self.file_extensions:
-                _logger.warning('No matching data format (i.e. file extension) for: ' + str(fn))
+            type_match = [x['name'] for x in self.dtypes
+                          if fnmatch(fn.name, x['filename_pattern'] or '')]
+            if len(type_match) == 0:
+                _logger.debug(f'{fn}: No matching dataset type in database')
                 continue
-            F[session_path].append(fn)
+            elif len(type_match) != 1:
+                _logger.debug(f'{fn}: Multiple matching dataset types in database\n'
+                              '"' + '", "'.join(type_match) + '"')
+                continue
+            F[session_path].append(fn.relative_to(session_path))
+            V[session_path].append(ver)
 
+        # For each unique session, make a separate POST request
+        records = []
         for session_path, files in F.items():
             # this is the generic relative path: subject/yyyy-mm-dd/NNN
-            get_alf_path()
             details = session_path_parts(session_path.as_posix(), as_dict=True, assert_valid=True)
             rel_path = PurePosixPath(details['subject'], details['date'], details['number'])
-            # try:
-            #     assert (str(rel_path) in str(fn))
-            # except AssertionError as e:
-            #     strerr = 'ALF folder mismatch: data is in wrong subject/date/number folder. \n'
-            #     strerr += ' Expected ' + str(gen_rel_path) + ' actual was ' + str(fn)
-            #     _logger.error(strerr)
-            #     raise e
-            # extract the relative path of the file
-            rel_path = Path(str(fn)[str(fn).find(str(session_path)):])
-            F[session_path].append(str(rel_path.relative_to(session_path).as_posix()))
-            file_sizes.append(fn.stat().st_size)
-            md5s.append(hashfile.md5(fn) if fn.stat().st_size < 1024 ** 3 else None)
-            _logger.info('Registering ' + str(fn))
+            file_sizes = [session_path.joinpath(fn).stat().st_size for fn in files]
+            # computing the md5 can be very long, so this is an option to skip if the file is
+            # bigger than a certain threshold
+            md5s = [hashfile.md5(session_path.joinpath(fn))
+                    if (max_md5_size is None or sz < max_md5_size) else None
+                    for fn, sz in zip(files, file_sizes)]
 
-        r_ = {'created_by': user,
-              'path': rel_path.as_posix(),
-              'filenames': F,
-              'hashes': md5s,
-              'filesizes': file_sizes,
-              'versions': [version.ibllib() for _ in F]
-              }
-        self.one.alyx.post('/register-file', data=r_)
+            _logger.info('Registering ' + str(files))
+
+            r_ = {'created_by': created_by or self.one.alyx.user,
+                  'path': rel_path.as_posix(),
+                  'filenames': [x.as_posix() for x in files],
+                  'hashes': md5s,
+                  'filesizes': file_sizes,
+                  'name': repository,
+                  'server_only': server_only,
+                  'default': default,
+                  'versions': V[session_path]}
+
+            # Add optional field
+            if details['lab']:
+                r_['labs'] = details['lab']
+            # If dry, store POST data, otherwise store resulting file records
+            records.append(r_ if dry else self.one.alyx.post('/register-file', data=r_))
+            # Log file names
+            _logger.info(f'ALYX REGISTERED DATA {"!DRY!" if dry else ""}: {rel_path}')
+            for p in files:
+                _logger.info(f"ALYX REGISTERED DATA: {p}")
+
+        return records[0] if len(F.keys()) == 1 else records
 
     def register_water_administration(self, subject, volume, **kwargs):
         """
@@ -407,7 +451,8 @@ class RegistrationClient:
 
         Returns
         -------
-        A water administration record
+        dict
+            A water administration record
 
         Raises
         ------
@@ -462,7 +507,8 @@ class RegistrationClient:
 
         Returns
         -------
-        An Alyx weight record
+        dict
+            An Alyx weight record
 
         Raises
         ------
@@ -490,30 +536,3 @@ class RegistrationClient:
                 'weight': float(f'{weight:.4g}'),  # Round to 4 s.f.
                 'user': user}
         return self.one.alyx.rest('weighings', 'create', data=wei_)
-
-
-def _register_bool(fn, file_list):
-    if isinstance(file_list, bool):
-        return file_list
-    if isinstance(file_list, str):
-        file_list = [file_list]
-    return any([str(fil) in fn for fil in file_list])
-
-
-def _glob_session(ses_path):
-    """
-    Glob for files to be registered on an IBL session
-
-    Parameters
-    ----------
-    ses_path : str, pathlib.Path
-        A session path
-
-    Returns
-    -------
-    A list of files to potentially be registered
-    """
-    fl = []
-    for gp in REGISTRATION_GLOB_PATTERNS:
-        fl.extend(list(Path(ses_path).glob(gp)))
-    return fl
