@@ -2,27 +2,31 @@
 The AlyxClient class contains methods for making remote Alyx REST queries and downloading remote
 files through Alyx.
 
-Examples:
-    alyx = AlyxClient(
-        username='test_user', password='TapetesBloc18',
-        base_url='https://test.alyx.internationalbrainlab.org')
+Examples
+--------
+>>> alyx = AlyxClient(
+...     username='test_user', password='TapetesBloc18',
+...     base_url='https://test.alyx.internationalbrainlab.org')
 
-    # List subjects
-    subjects = alyx.rest('subjects', 'list')
+List subjects
 
-    # Create a subject
-    record = {
-        'nickname': nickname,
-        'responsible_user': 'olivier',
-        'birth_date': '2019-06-15',
-        'death_date': None,
-        'lab': 'cortexlab',
-    }
-    new_subj = alyx.rest('subjects', 'create', data=record)
+>>> subjects = alyx.rest('subjects', 'list')
 
-    # Download a remote file, given a local path
-    url = 'zadorlab/Subjects/flowers/2018-07-13/1/channels.probe.npy'
-    local_path = self.alyx.download_file(url)
+Create a subject
+
+>>> record = {
+...     'nickname': nickname,
+...     'responsible_user': 'olivier',
+...     'birth_date': '2019-06-15',
+...     'death_date': None,
+...     'lab': 'cortexlab',
+... }
+... new_subj = alyx.rest('subjects', 'create', data=record)
+
+Download a remote file, given a local path
+
+>>> url = 'zadorlab/Subjects/flowers/2018-07-13/1/channels.probe.npy'
+>>> local_path = alyx.download_file(url)
 """
 import json
 import logging
@@ -55,26 +59,45 @@ _logger = logging.getLogger(__name__)
 
 
 def _cache_response(method):
-    """
-    Decorator for the generic request method; caches the result of the query and on subsequent
-    calls, returns cache instead of hitting the database
-    :param method: function to wrap (i.e. AlyxClient._generic_request)
-    :return: handle to wrapped method
+    """Decorator for the generic request method
+
+    Caches the result of the query and on subsequent calls, returns cache instead of hitting the
+    database
+
+    Parameters
+    ----------
+    method : function
+        Function to wrap (i.e. AlyxClient._generic_request)
+
+    Returns
+    -------
+    function
+        Handle to wrapped method
     """
 
     @functools.wraps(method)
     def wrapper_decorator(alyx_client, *args, expires=None, clobber=False, **kwargs):
         """
         REST caching wrapper
-        :param alyx_client: an instance of the AlyxClient class
-        :param args: positional arguments for applying to wrapped function
-        :param expires: an optional timedelta for how long cached response is valid.  If True,
-        the cached response will not be used on subsequent calls.  If None, the default expiry
-        is applied.
-        :param clobber: If True any existing cached response is overwritten
-        :param kwargs: keyword arguments for applying to wrapped function
-        :return: the REST response JSON either from cached file or directly from remote
-        endpoint
+
+        Parameters
+        ----------
+        alyx_client : AlyxClient
+            An instance of the AlyxClient class
+        args : any
+            Positional arguments for applying to wrapped function
+        expires : bool
+            An optional timedelta for how long cached response is valid.  If True, the cached
+            response will not be used on subsequent calls.  If None, the default expiry is applied.
+        clobber : bool
+            If True any existing cached response is overwritten
+        kwargs : any
+            Keyword arguments for applying to wrapped function
+
+        Returns
+        -------
+        dict
+            The REST response JSON either from cached file or directly from remote
         """
         expires = expires or alyx_client.default_expiry
         mode = (alyx_client.cache_mode or '').lower()
@@ -117,14 +140,30 @@ def _cache_response(method):
 class _PaginatedResponse(Mapping):
     """
     This class allows to emulate a list from a paginated response.
-    Provides cache functionality
-    PaginatedResponse(alyx, response)
+    Provides cache functionality.
+
+    Examples
+    --------
+    >>> r = _PaginatedResponse(client, response)
     """
 
-    def __init__(self, alyx, rep):
+    def __init__(self, alyx, rep, cache_args=None):
+        """
+        A paginated response cache object
+
+        Parameters
+        ----------
+        alyx : AlyxClient
+            An instance of an AlyxClient associated with the REST response
+        rep : dict
+            A paginated REST response JSON dictionary
+        cache_args : dict
+            A dict of kwargs to pass to _cache_response decorator upon subsequent requests
+        """
         self.alyx = alyx
         self.count = rep['count']
         self.limit = len(rep['results'])
+        self._cache_args = cache_args or {}
         # store URL without pagination query params
         self.query = rep['next']
         # init the cache, list with None with count size
@@ -147,8 +186,12 @@ class _PaginatedResponse(Mapping):
     def populate(self, idx):
         offset = self.limit * math.floor(idx / self.limit)
         query = update_url_params(self.query, {'limit': self.limit, 'offset': offset})
-        res = self.alyx._generic_request(requests.get, query)
-        for i, r in enumerate(res['results']):
+        res = self.alyx._generic_request(requests.get, query, **self._cache_args)
+        if self.count != res['count']:
+            warnings.warn(
+                f'remote results for {urllib.parse.urlsplit(query).path} endpoint changed; '
+                f'results may be inconsistent', RuntimeWarning)
+        for i, r in enumerate(res['results'][:self.count - offset]):
             self._cache[i + offset] = res['results'][i]
 
     def __iter__(self):
@@ -168,15 +211,16 @@ def update_url_params(url: str, params: dict) -> str:
 
     Returns
     -------
+    str
         A new URL with said parameters updated
 
     Examples
     -------
-        update_url_params('website.com/?q=', {'pg': 5})
-        'website.com/?pg=5'
+    >>> update_url_params('website.com/?q=', {'pg': 5})
+    'website.com/?pg=5'
 
-        update_url_params('website.com?q=xxx', {'pg': 5, 'foo': ['bar', 'baz']})
-        'website.com?q=xxx&pg=5&foo=bar&foo=baz'
+    >>> update_url_params('website.com?q=xxx', {'pg': 5, 'foo': ['bar', 'baz']})
+    'website.com?q=xxx&pg=5&foo=bar&foo=baz'
     """
     # Remove percent-encoding
     url = urllib.parse.unquote(url)
@@ -193,13 +237,20 @@ def update_url_params(url: str, params: dict) -> str:
 
 def http_download_file_list(links_to_file_list, **kwargs):
     """
-    Downloads a list of files from the flat Iron from a list of links.
-    Same options behaviour as http_download_file
+    Downloads a list of files from a remote HTTP server from a list of links.
+    Same options behaviour as http_download_file.
 
-    :param links_to_file_list: list of http links to files.
-    :type links_to_file_list: list
+    Parameters
+    ----------
+    links_to_file_list : list
+        List of http links to files
+    kwargs : any
+        Optional arguments to pass to http_download_file
 
-    :return: (list) a list of the local full path of the downloaded files.
+    Returns
+    -------
+    list of pathlib.Path
+        A list of the local full path of the downloaded files.
     """
     file_names_list = []
     for link_str in links_to_file_list:
@@ -210,27 +261,33 @@ def http_download_file_list(links_to_file_list, **kwargs):
 def http_download_file(full_link_to_file, chunks=None, *, clobber=False, silent=False,
                        username='', password='', cache_dir='', return_md5=False, headers=None):
     """
-    :param full_link_to_file: http link to the file.
-    :type full_link_to_file: str
-    :param chunks: chunks to download
-    :type chunks: tuple of ints
-    :param clobber: [False] If True, force overwrite the existing file.
-    :type clobber: bool
-    :param username: [''] authentication for password protected file server.
-    :type username: str
-    :param password: [''] authentication for password protected file server.
-    :type password: str
-    :param cache_dir: [''] directory in which files are cached; defaults to user's
-     Download directory.
-    :type cache_dir: str
-    :param return_md5: if true an MD5 hash of the file is additionally returned
-    :type return_md5: bool
-    :param: headers: [{}] additional headers to add to the request (auth tokens etc..)
-    :type headers: dict
-    :param: silent: [False] suppress download progress bar
-    :type silent: bool
+    Download a file from a remote HTTP server.
 
-    :return: (str) a list of the local full path of the downloaded files.
+    Parameters
+    ----------
+    full_link_to_file : str
+        HTTP link to the file
+    chunks : tuple of ints
+        Chunks to download
+    clobber : bool
+        If True, force overwrite the existing file
+    silent : bool
+        If True, suppress download progress bar
+    username : str
+        User authentication for password protected file server
+    password : str
+        Password authentication for password protected file server
+    cache_dir : str, pathlib.Path
+        Directory in which files are cached; defaults to user's Download directory
+    return_md5 : bool
+        If True an MD5 hash of the file is additionally returned
+    headers : list of dicts
+        Additional headers to add to the request (auth tokens etc.)
+
+    Returns
+    -------
+    pathlib.Path
+        The full file path of the downloaded file
     """
     if not full_link_to_file:
         return ''
@@ -306,14 +363,19 @@ def http_download_file(full_link_to_file, chunks=None, *, clobber=False, silent=
     return (file_name, md5.hexdigest()) if return_md5 else file_name
 
 
-def file_record_to_url(file_records):
+def file_record_to_url(file_records) -> list:
     """
     Translate a Json dictionary to an usable http url for downloading files.
 
-    :param file_records: json containing a 'data_url' field
-    :type file_records: dict
+    Parameters
+    ----------
+    file_records : dict
+        JSON containing a 'data_url' field
 
-    :return: urls: (list) a list of strings representing full data urls
+    Returns
+    -------
+    list of str
+        A list of full data urls
     """
     urls = []
     for fr in file_records:
@@ -322,14 +384,19 @@ def file_record_to_url(file_records):
     return urls
 
 
-def dataset_record_to_url(dataset_record):
+def dataset_record_to_url(dataset_record) -> list:
     """
     Extracts a list of files urls from a list of dataset queries.
 
-    :param dataset_record: dataset Json from a rest request.
-    :type dataset_record: list
+    Parameters
+    ----------
+    dataset_record : list, dict
+        Dataset JSON from a REST request
 
-    :return: (list) a list of strings representing files urls corresponding to the datasets records
+    Returns
+    -------
+    list of str
+        A list of file urls corresponding to the datasets records
     """
     urls = []
     if isinstance(dataset_record, dict):
@@ -341,30 +408,42 @@ def dataset_record_to_url(dataset_record):
 
 class AlyxClient():
     """
-    Class that implements simple GET/POST wrappers for the Alyx REST API
-    http://alyx.readthedocs.io/en/latest/api.html  # FIXME old link
+    Class that implements simple GET/POST wrappers for the Alyx REST API.
+    See https://openalyx.internationalbrainlab.org/docs
     """
     _token = None
     _headers = None  # Headers for REST requests only
+    """str: The Alyx username"""
     user = None
+    """str: The Alyx database URL"""
     base_url = None
 
     def __init__(self, base_url=None, username=None, password=None,
                  cache_dir=None, silent=False, cache_rest='GET', stay_logged_in=True):
         """
-        Create a client instance that allows to GET and POST to the Alyx server
-        For oneibl, constructor attempts to authenticate with credentials in params.py
-        For standalone cases, AlyxClient(username='', password='', base_url='')
+        Create a client instance that allows to GET and POST to the Alyx server.
+        For One, constructor attempts to authenticate with credentials in params.py.
+        For standalone cases, AlyxClient(username='', password='', base_url='').
 
-        :param username: Alyx database user
-        :param password: Alyx database password
-        :param base_url: Alyx server address, including port and protocol
-        :param cache_rest: which type of http method to apply cache to; if '*', all requests are
-        cached.
+        Parameters
+        ----------
+        base_url : str
+            Alyx server address, including port and protocol
+        username : str
+            Alyx database user
+        password : str
+            Alyx database password
+        cache_dir : str, pathlib.Path
+            The default download location
+        silent : bool
+            If true, user prompts and progress bars are suppressed
+        cache_rest : str
+            Which type of http method to apply cache to; if '*', all requests are cached
+        stay_logged_in : bool
+            If true, auth token is cached
         """
         self.silent = silent
         self._par = one.params.get(client=base_url, silent=self.silent)
-        # TODO Pass these to `params.get` and have it deal with setup defaults
         self.base_url = base_url or self._par.ALYX_URL
         self._par = self._par.set('CACHE_DIR', cache_dir or self._par.CACHE_DIR)
         self.authenticate(username, password, cache_token=stay_logged_in)
@@ -381,22 +460,20 @@ class AlyxClient():
 
     @property
     def rest_schemes(self):
-        """Delayed fetch of rest schemes speeds up instantiation"""
+        """dict: The REST endpoints and their parameters"""
+        # Delayed fetch of rest schemes speeds up instantiation
         if not self._rest_schemes:
             self._rest_schemes = self.get('/docs', expires=timedelta(weeks=1))
         return self._rest_schemes
 
     @property
     def cache_dir(self):
+        """pathlib.Path: The location of the downloaded file cache"""
         return Path(self._par.CACHE_DIR)
 
+    @property
     def is_logged_in(self):
-        """Check if user logged into Alyx database
-
-        Returns
-        -------
-            True if user is authenticated
-        """
+        """bool: Check if user logged into Alyx database; True if user is authenticated"""
         return self._token and self.user and self._headers and 'Authorization' in self._headers
 
     def list_endpoints(self):
@@ -433,10 +510,11 @@ class AlyxClient():
         elif r and r.status_code == 204:
             return
         else:
-            if not self.silent:
-                _logger.error(self.base_url + rest_query)
-                _logger.error(r.text)
-            raise (requests.HTTPError(r))
+            try:
+                message = json.loads(r.text).get('detail')
+            except json.decoder.JSONDecodeError:
+                message = r.text
+            raise requests.HTTPError(r.status_code, rest_query, message, response=r)
 
     def authenticate(self, username=None, password=None, cache_token=True, force=False):
         """
@@ -473,7 +551,7 @@ class AlyxClient():
         if password is None:
             password = getattr(self._par, 'ALYX_PWD', None)
         if password is None and not self.silent:
-            password = getpass('Enter Alyx password:')
+            password = getpass(f'Enter Alyx password for "{username}":')
         try:
             credentials = {'username': username, 'password': password}
             rep = requests.post(self.base_url + '/auth-token', data=credentials)
@@ -482,11 +560,19 @@ class AlyxClient():
                 f"Can't connect to {self.base_url}.\n" +
                 "Check your internet connections and Alyx database firewall"
             )
-        # Assign token or raise exception on internal server error
-        self._token = rep.json() if rep.ok else rep.raise_for_status()
-        if not (list(self._token.keys()) == ['token']):
-            _logger.error(rep)
-            raise Exception('Alyx authentication error. Check your credentials')
+        # Assign token or raise exception on auth error
+        if rep.ok:
+            self._token = rep.json()
+            assert list(self._token.keys()) == ['token']
+        else:
+            if rep.status_code == 400:  # Auth error; re-raise with details
+                redacted = '*' * len(credentials['password']) if credentials['password'] else None
+                message = ('Alyx authentication failed with credentials: '
+                           f'user = {credentials["username"]}, password = {redacted}')
+                raise requests.HTTPError(rep.status_code, rep.url, message, response=rep)
+            else:
+                rep.raise_for_status()
+
         self._headers = {
             'Authorization': 'Token {}'.format(list(self._token.values())[0]),
             'Accept': 'application/json'}
@@ -506,7 +592,7 @@ class AlyxClient():
         """Log out from Alyx
         Deletes the cached authentication token for the currently logged-in user
         """
-        if not self.is_logged_in():
+        if not self.is_logged_in:
             return
         par = one.params.get(client=self.base_url, silent=True)
         username = self.user
@@ -531,21 +617,37 @@ class AlyxClient():
         Sends a DELETE request to the Alyx server. Will raise an exception on any status_code
         other than 200, 201.
 
-        :param rest_query: examples:
-         '/weighings/c617562d-c107-432e-a8ee-682c17f9e698'
-         'https://test.alyx.internationalbrainlab.org/weighings/c617562d-c107-432e-a8ee-682c17f9e698'.
-        :type rest_query: str
+        Parameters
+        ----------
+        rest_query : str
+            A REST query string either as a relative URL path complete URL
 
-        :return: (dict/list) json interpreted dictionary from response
+        Returns
+        -------
+        JSON interpreted dictionary from response
+
+        Examples
+        --------
+        >>> AlyxClient.delete('/weighings/c617562d-c107-432e-a8ee-682c17f9e698')
+        >>> AlyxClient.delete(
+        ...     'https://alyx.example.com/endpoint/c617562d-c107-432e-a8ee-682c17f9e698')
         """
         return self._generic_request(requests.delete, rest_query)
 
     def download_file(self, url, **kwargs):
         """
         Downloads a file on the Alyx server from a file record REST field URL
-        :param url: full url(s) of the file(s)
-        :param kwargs: webclient.http_download_file parameters
-        :return: local path(s) of downloaded file(s)
+
+        Parameters
+        ----------
+        url : str, list
+            Full url(s) of the file(s)
+        kwargs : Any
+            WebClient.http_download_file parameters
+
+        Returns
+        -------
+        Local path(s) of downloaded file(s)
         """
         if isinstance(url, str):
             url = self._validate_file_url(url)
@@ -560,7 +662,14 @@ class AlyxClient():
             password=self._par.HTTP_DATA_SERVER_PWD,
             **kwargs
         )
-        return download_fcn(url, **pars)
+        try:
+            files = download_fcn(url, **pars)
+        except HTTPError as ex:
+            if ex.code == 401:
+                ex.msg += (' - please check your HTTP_DATA_SERVER_LOGIN and '
+                           'HTTP_DATA_SERVER_PWD ONE params, or username/password kwargs')
+            raise ex
+        return files
 
     def download_cache_tables(self):
         """Downloads the Alyx cache tables to the local data cache directory
@@ -571,7 +680,7 @@ class AlyxClient():
         """
         # query the database for the latest cache; expires=None overrides cached response
         self.cache_dir.mkdir(exist_ok=True)
-        if not self.is_logged_in():
+        if not self.is_logged_in:
             self.authenticate()
         with tempfile.TemporaryDirectory(dir=self.cache_dir) as tmp:
             file = http_download_file(f'{self.base_url}/cache.zip',
@@ -600,10 +709,10 @@ class AlyxClient():
 
         Examples
         --------
-            url = self._validate_file_url('https://webserver.net/path/to/file')
-            'https://webserver.net/path/to/file'
-            url = self._validate_file_url('path/to/file')
-            'https://webserver.net/path/to/file'
+        >>> url = self._validate_file_url('https://webserver.net/path/to/file')
+        'https://webserver.net/path/to/file'
+        >>> url = self._validate_file_url('path/to/file')
+        'https://webserver.net/path/to/file'
         """
         if url.startswith('http'):  # A full URL
             assert url.startswith(self._par.HTTP_DATA_SERVER), \
@@ -635,18 +744,25 @@ class AlyxClient():
         Sends a GET request to the Alyx server. Will raise an exception on any status_code
         other than 200, 201.
         For the dictionary contents and list of endpoints, refer to:
-        https://alyx.internationalbrainlab.org/docs
+        https://openalyx.internationalbrainlab.org/docs
 
-        :param rest_query: example: '/sessions?user=Hamish'.
-        :type rest_query: str
+        Parameters
+        ----------
+        rest_query : str
+            A REST URL path, e.g. '/sessions?user=Hamish'
+        kwargs : any
+            Optional arguments to pass to _generic_request and _cache_response decorator
 
-        :return: (dict/list) json interpreted dictionary from response
+        Returns
+        -------
+        JSON interpreted dictionary from response
         """
         rep = self._generic_request(requests.get, rest_query, **kwargs)
         _logger.debug(rest_query)
         if isinstance(rep, dict) and list(rep.keys()) == ['count', 'next', 'previous', 'results']:
             if len(rep['results']) < rep['count']:
-                rep = _PaginatedResponse(self, rep)
+                cache_args = {k: v for k, v in kwargs.items() if k in ('clobber', 'expires')}
+                rep = _PaginatedResponse(self, rep, cache_args)
             else:
                 rep = rep['results']
         return rep
@@ -655,15 +771,20 @@ class AlyxClient():
         """
         Sends a PATCH request to the Alyx server.
         For the dictionary contents, refer to:
-        https://alyx.internationalbrainlab.org/docs
+        https://openalyx.internationalbrainlab.org/docs
 
-        :param rest_query: (required)the endpoint as full or relative URL
-        :type rest_query: str
-        :param data: json encoded string or dictionary (cf.requests)
-        :type data: None, dict or str
-        :param files: dictionary / tuple (cf.requests)
+        Parameters
+        ----------
+        rest_query : str
+            The endpoint as full or relative URL
+        data : dict, str
+            JSON encoded string or dictionary (c.f. requests)
+        files : dict, tuple
+            Files to attach (c.f. requests)
 
-        :return: response object
+        Returns
+        -------
+        Response object
         """
         return self._generic_request(requests.patch, rest_query, data=data, files=files)
 
@@ -671,15 +792,20 @@ class AlyxClient():
         """
         Sends a POST request to the Alyx server.
         For the dictionary contents, refer to:
-        https://alyx.internationalbrainlab.org/docs
+        https://openalyx.internationalbrainlab.org/docs
 
-        :param rest_query: (required)the endpoint as full or relative URL
-        :type rest_query: str
-        :param data: dictionary or json encoded string
-        :type data: None, dict or str
-        :param files: dictionary / tuple (cf.requests)
+        Parameters
+        ----------
+        rest_query : str
+            The endpoint as full or relative URL
+        data : dict, str
+            JSON encoded string or dictionary (c.f. requests)
+        files : dict, tuple
+            Files to attach (c.f. requests)
 
-        :return: response object
+        Returns
+        -------
+        Response object
         """
         return self._generic_request(requests.post, rest_query, data=data, files=files)
 
@@ -687,15 +813,21 @@ class AlyxClient():
         """
         Sends a PUT request to the Alyx server.
         For the dictionary contents, refer to:
-        https://alyx.internationalbrainlab.org/docs
+        https://openalyx.internationalbrainlab.org/docs
 
-        :param rest_query: (required)the endpoint as full or relative URL
-        :type rest_query: str
-        :param data: dictionary or json encoded string
-        :type data: None, dict or str
-        :param files: dictionary / tuple (cf.requests)
+        Parameters
+        ----------
+        rest_query : str
+            The endpoint as full or relative URL
+        data : dict, str
+            JSON encoded string or dictionary (c.f. requests)
+        files : dict, tuple
+            Files to attach (c.f. requests)
 
-        :return: response object
+        Returns
+        -------
+        requests.Response
+            Response object
         """
         return self._generic_request(requests.put, rest_query, data=data, files=files)
 
@@ -708,24 +840,38 @@ class AlyxClient():
 
         Example REST endpoint with all actions:
 
-            client.rest('subjects', 'list')
-            client.rest('subjects', 'list', field_filter1='filterval')
-            client.rest('subjects', 'create', data=sub_dict)
-            client.rest('subjects', 'read', id='nickname')
-            client.rest('subjects', 'update', id='nickname', data=sub_dict)
-            client.rest('subjects', 'partial_update', id='nickname', data=sub_dict)
-            client.rest('subjects', 'delete', id='nickname')
-            client.rest('notes', 'create', data=nd, files={'image': open(image_file, 'rb')})
+        >>> client = AlyxClient()
+        >>> client.rest('subjects', 'list')
+        >>> client.rest('subjects', 'list', field_filter1='filterval')
+        >>> client.rest('subjects', 'create', data=sub_dict)
+        >>> client.rest('subjects', 'read', id='nickname')
+        >>> client.rest('subjects', 'update', id='nickname', data=sub_dict)
+        >>> client.rest('subjects', 'partial_update', id='nickname', data=sub_dict)
+        >>> client.rest('subjects', 'delete', id='nickname')
+        >>> client.rest('notes', 'create', data=nd, files={'image': open(image_file, 'rb')})
 
-        :param url: endpoint name
-        :param action: 'list', 'create', 'read', 'update', 'partial_update', 'delete'
-        :param id: lookup string for actions 'read', 'update', 'partial_update', and 'delete'
-        :param data: data dictionary for actions 'update', 'partial_update' and 'create'
-        :param files: if file upload
-        :param no_cache: if true the `list` and `read` actions are performed without caching
-        :param ``**kwargs``: filter as per the Alyx REST documentation
-            cf. https://alyx.internationalbrainlab.org/docs/
-        :return: list of queried dicts ('list') or dict (other actions)
+        Parameters
+        ----------
+        url : str
+            Endpoint name
+        action : str
+            One of 'list', 'create', 'read', 'update', 'partial_update', 'delete'
+        id : str
+            Lookup string for actions 'read', 'update', 'partial_update', and 'delete'
+        data : dict
+            Data dictionary for actions 'update', 'partial_update' and 'create'
+        files : dict, tuple
+            Option file(s) to upload
+        no_cache : bool
+            If true the `list` and `read` actions are performed without caching
+        kwargs : any
+            Filters as per the Alyx REST documentation
+            cf. https://openalyx.internationalbrainlab.org/docs/
+
+        Returns
+        -------
+        list, dict
+            List of queried dicts ('list') or dict (other actions)
         """
         # if endpoint is None, list available endpoints
         if not url:
@@ -837,7 +983,8 @@ class AlyxClient():
 
         Returns
         -------
-        Written data dict
+        dict
+            Written data dict
         """
         self._check_inputs(endpoint)
         # Prepare data to patch
@@ -872,11 +1019,13 @@ class AlyxClient():
 
         Returns
         -------
-        New patched json field contents as dict
+        dict
+            New patched json field contents as dict
 
         Examples
         --------
-        one.alyx.json_field_update("sessions", "eid_str", "extended_qc", {"key": value})
+        >>> client = AlyxClient()
+        >>> client.json_field_update("sessions", "eid_str", "extended_qc", {"key": "value"})
         """
         self._check_inputs(endpoint)
         # Load current json field contents
@@ -906,19 +1055,24 @@ class AlyxClient():
             key: str = None
     ) -> Optional[dict]:
         """json_field_remove_key
-        Will remove inputted key from json field dict and reupload it to Alyx.
+        Will remove inputted key from json field dict and re-upload it to Alyx.
         Needs endpoint, uuid and json field name
 
-        :param endpoint: endpoint to hit, defaults to None
-        :type endpoint: str, optional
-        :param uuid: uuid or lookup name for endpoint
-        :type uuid: str, optional
-        :param field_name: json field name of object, defaults to None
-        :type field_name: str, optional
-        :param key: key name of dictionary inside object, defaults to None
-        :type key: str, optional
-        :return: returns new content of json field
-        :rtype: dict
+        Parameters
+        ----------
+        endpoint : str
+            Endpoint to hit, defaults to None
+        uuid : str
+            UUID or lookup name for endpoint
+        field_name : str
+            JSON field name of object, defaults to None
+        key : str
+            Key name of dictionary inside object, defaults to None
+
+        Returns
+        -------
+        dict
+            New content of json field
         """
         self._check_inputs(endpoint)
         current = self.rest(endpoint, "read", id=uuid)[field_name]
@@ -951,5 +1105,6 @@ class AlyxClient():
         return _[field_name]
 
     def clear_rest_cache(self):
+        """Clear all REST response cache files for the base url"""
         for file in one.params.get_rest_dir(self.base_url).glob('*'):
             file.unlink()
